#include <FL/fl_ask.H>
#include <config.h>
#include <ChiliLog.h>
#include <ChiliSession.h>
#include <ChiliProject.h>
#include <FL/Fl_Preferences.H>
#include <FL/filename.H>
#include <numtostr.h>
using namespace std;
ChiliSession::ChiliSession() {
	unsigned int i;
	log("ChiliSession::ChiliSession()");
	lasterror=SUCCESS;
	for (i=0;i<MAXPROJECTS;i++) {
		projectlist[i]=NULL;
	}

	this->ChiliSessionInit();
}

ChiliSession::~ChiliSession() {
	unsigned int i;
	log("ChiliSession::~ChiliSession()");
	ChiliProject * cp;
	for (i=0;i<MAXPROJECTS;i++) {
		cp=projectlist[i];
		projectlist[i]=NULL;
		if (cp==NULL) continue;
		delete cp;
	}
	uisession=NULL;
	if (sysprefs!=NULL) {
		delete sysprefs;
	}
	if (userprefs!=NULL) {
		delete userprefs;
	}
}
void ChiliSession::SetUISession(ChiliUISession * p_uisession) {
	uisession=p_uisession;
}
ChiliUISession * ChiliSession::GetUISession() {
	return uisession;
}
string	ChiliSession::GetTempDir() {
	return tempdir;
}
Fl_Preferences * ChiliSession::GetUserPrefs() {
	return userprefs;
}
Fl_Preferences * ChiliSession::GetSysPrefs() {
	return sysprefs;
}
void 	ChiliSession::ChiliSessionInit() {
	uisession=NULL;
	char buffer[FL_PATH_MAX];
	systemhomedir=*new string("");
	userhomedir=*new string("");
	tempdir=*new string("");

	sysprefs=new Fl_Preferences(Fl_Preferences::SYSTEM,"CompFu","ChiliAudio" );
	userprefs=new Fl_Preferences(Fl_Preferences::USER, "CompFu","ChiliAudio" );

	/* Attempt to init missing stuff in the session */
	printf("ChiliSession::ChiliSessionInit()\n");

	sysprefs->getUserdataPath(buffer,FL_PATH_MAX);
	systemhomedir=buffer;
	userprefs->getUserdataPath(buffer,FL_PATH_MAX);
	userhomedir=buffer;
	printf("System config dir is %s\n",systemhomedir.c_str());
	printf("User config dir is %s\n",userhomedir.c_str());

	/* Attempt to find temp dir */
	if (!(userprefs->entryExists("tempdir"))) {
		userprefs->set("tempdir",userhomedir.c_str());
	}
	userprefs->get("tempdir",buffer,userhomedir.c_str(),FL_PATH_MAX);
	tempdir=buffer;
	printf("Temp dir is %s\n",tempdir.c_str());
	return;
}

unsigned int ChiliSession::GetFreeSlot() {
	unsigned int slotnum=0;
	log("ChiliSession::GetFreeSlot");
	while (slotnum<MAXPROJECTS) {
		if (projectlist[slotnum]==NULL) return slotnum;
		slotnum++;
	}
	return slotnum;
}

unsigned int ChiliSession::CreateProject(string filename) {
	/* NULL filename creates new empty ChiliProject */
	int freeslot;
	log("ChiliSession::CreateProject");
	freeslot=GetFreeSlot();
	if (freeslot>=MAXPROJECTS) {
		lasterror=ERROR_NO_FREE_SLOTS;
		return MAXPROJECTS;
	}
	ChiliProject * proj=NULL;
	try {
		proj=new ChiliProject(this,freeslot,filename);
	} catch (int e) {
		log("ChiliSession::Failed creating project, error %s",numtostr(e).c_str());
		throw e;
	}
	projectlist[freeslot]=proj;
	return freeslot;
}
ChiliProject * ChiliSession::GetProjectBySlot(const unsigned int projid) {
	log("ChiliSession::GetProjectBySlot");
	return projectlist[projid];
}

unsigned int ChiliSession::CloseProject(const unsigned int proj_id,bool confirm) {
	log("ChiliSession::CloseProject");
	if (proj_id>=MAXPROJECTS) {
		/* project already closed, so after doing nothing
		   closing it succeeded. */
		return SUCCESS;
	}
	ChiliProject * proj=projectlist[proj_id];
	if (proj==NULL) {
		/* No project in this slot, so it is already closed. */
		return SUCCESS;
	}
	delete proj;
	return SUCCESS;
}

